/* sample code to run ZHEEV from c
 
   Created by G.P. Brandino for the course 
   "P1.2 Introduction to computer architecture"
   for the Master in High Performance Computing @SISSA/ICTP */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <plasma.h>

#include "FillMatrixplasma.c"
#include "cptimer.c"

int main (int argc, char* argv[])
{
  if (argc!=2)
     {
     printf("\n");
     printf("%-20s \n\n", "Sample ZHEEV code.") ;
     printf("%-8s \n %-4s %-20s\n", "usage:", argv[0], "{matrix size}");
     return 0; 
     }
  int  n=(int) atoi (argv[1]);

  // Allocation of the Hermitian Matrix
  PLASMA_Complex64_t  *A;
  A = (PLASMA_Complex64_t *) malloc(sizeof(PLASMA_Complex64_t)*n*n);

  // Initialization of the sample matrix
  FillMatrix( A, n);

  // Print the Initliazed Hermitian matrix
  FILE *outputmat = fopen("matrix.dat","wb");
  int j;
  for ( j=0; j<n; j++)
      {
      fwrite (A, sizeof(PLASMA_Complex64_t), n*n, outputmat);
      }
  fclose(outputmat); 

  // Allocation of temporary variables for PLASMA call to ZHEEV workspace
  int LDA= n, info=0;
  double w[n] ;
  PLASMA_desc *descT ; 
  PLASMA_Complex64_t  *Q;
  int LDQ= n;
  Q = (PLASMA_Complex64_t *) malloc(sizeof(PLASMA_Complex64_t)*n);

  // Initialize Plasma to 0 so PLASMA_NUM_THREADS environment variable is used
  PLASMA_Init(0);
  // Allocate workspace for the diagonalization call
  PLASMA_Alloc_Workspace_zheev( n, n,  &descT); 
  
  // Time measuring for the eingenvalues computation
  double start = cclock_(); 	
  info = PLASMA_zheev(PlasmaNoVec, PlasmaUpper, n, A, LDA, w, descT, Q, LDQ) ; 
  double end =  cclock_();

  printf("%-14s %f %-9s \n\n", "Done! It took", end-start , "seconds");
  FILE *output = fopen("eigenvalues.dat","wb");
  int i;
  for ( i=0; i<n; i++)
      {
      fwrite (w, sizeof(PLASMA_Complex64_t), n, output);
      }
  fclose(output);
 
  // Memory Deallocation and Cleaning
  free(descT);
  free(A) ;
  free(Q) ;
  PLASMA_Finalize();
 
  return 0;
}
