/* 
 * Program to compute eigenvalues of an Hermitian matrix using 
 * the LAPACKE interface of the subroutine zheev
 *
 * Code created for hands-on based evaluation for the course
 * "P1.2 Introduction to Computer Architecture for HPC"
 * for the Master in High Performance Computing @SISSA/ICTP
 *
 * Developed by: G.P. Brandino (brandino@sissa.it) and M. Atambo (matambo@ictp.it) 
 * Reviewd by: I. Girotto ( igirotto@ictp.it )
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "utils_lapacke.h"

int main( int argc, char *argv[] ){

  int n = 0;
  FILE* fp;
  double start, end;

  // Declaration and initialization of temporary variables for LAPACKE_zheev workspace
  int lda = 0, info = 0; 
  double *w;

  // Declaration of double pointer for referring HErmitian Matrix A
  MKL_Complex16 *A;

  // Minimal instructions for reading the matrix size from standard input
  if( argc !=2 ){
     printf("\n");
     printf("%-20s \n\n", "Sample ZHEEV code.") ;
     printf("%-8s %-4s %-20s", "usage:", argv[0], "{matrix size}");
     return 0; 
  }

  // Allocation of the Hermitian Matrix 
  n = (int) atoi( argv[1] );

  // A = (MKL_Complex16 *) malloc( sizeof(MKL_Complex16) * n * n );
  A = (MKL_Complex16 *) mkl_malloc( sizeof(MKL_Complex16) * n * n, 64 );

  // Initialization of the sample matrix
  //  FillMatrix_lapacke( A, n );

  // Print the Initliazed Hermitian matrix
  fp = fopen( "matrix.dat", "w+b" );
  fwrite( A, sizeof(MKL_Complex16), n*n, fp );
  fclose(fp);
   
  /*
   * Allocation of the workspace accordingly to the LAPACKE interface documentation.
   */
  lda = n;
  w = (double*) malloc( n * sizeof(double) ); 

  // Time measuring for the eingenvalues computation
  start = cclock();
  info = LAPACKE_zheev( LAPACK_ROW_MAJOR, 'V', 'U', n, A, lda, w );
  end =  cclock();

  fprintf( stdout, "\nTime to compute eigenvalues for an Hermitian matrix of dimension %d using LAPACK function LAPACKE_zheev: %.3g (seconds)\n", n, end - start );

  // Write the eigenvalues of the Hermitian matrix
  fp = fopen( "eigenvalues.dat", "w+b" );
  fwrite( w, sizeof(MKL_Complex16), n, fp );
  fclose(fp);
 
  mkl_free(A);
  free(w);

  return 0;
}
