/* 
 * Routine to generate a random Hermitian matrix in column-major order (FORTRAN-style)
 *
 * Code created for hands-on based evaluation for the course
 * "P1.2 Introduction to Computer Architecture for HPC"
 * for the Master in High Performance Computing @SISSA/ICTP
 *
 * Developed by: G.P. Brandino (brandino@sissa.it) and M. Atambo (matambo@ictp.it) 
 * Reviewd by: I. Girotto ( igirotto@ictp.it )
 *
 */

#include <stdlib.h>
#include <math.h>
#include "utils.h"

void FillMatrix( COMPLEX* A, int n ){

  double temp, temp1;
  int i,j ;

  srand( time(NULL) );

  /* for calling LAPACK zheev FORTRAN interface from C-style code the 
   * Hermitian matrix must be filled in column-major order
   */ 
  for ( i = 0; i < n; i++ ){
    for ( j = 0; j < i; j++ ){
      
      if( i == j ){
             
	temp = rand() / ( (double) RAND_MAX );
	A[ j * n + i ].real = temp;
	A[ j * n + i ].imag = 0.0;
      }
      else{
	
	temp = rand() / ( (double) RAND_MAX );
	temp1 = rand() / ( (double) RAND_MAX );
	A[ j * n + i ].real = temp;
	A[ j * n + i ].imag = temp1;
	A[ i * n + j ].real = temp;
	A[ i * n + j ].imag = -temp1;
      }
    }
  }
}
