PROGRAM Diagonalize_random_lapack
  !
  ! Program to compute eigenvalues of an Hermitian matrix using LAPACK subroutine zheev
  ! 
  ! Code created for hands-on based evaluation for the course
  ! "P1.2 Introduction to Computer Architecture for HPC"
  ! for the Master in High Performance Computing @SISSA/ICTP
  !
  ! Developed by: G.P. Brandino (brandino@sissa.it) and M. Atambo (matambo@ictp.it) 
  ! Reviewd by: I. Girotto ( igirotto@ictp.it )
  !

  USE FillHermitian, ONLY : fillMatrix

  IMPLICIT NONE

  INTEGER  ::  arg_count, out_unit = 98, reclen
  CHARACTER( len = 8 ) :: arg1 

  ! Definition of the Hermitian Matrix related variables types 
  INTEGER :: n
  COMPLEX * 16, DIMENSION( : , : ), ALLOCATABLE :: A 

  ! Temporary variables for LAPACK call to zheev workspace  
  INTEGER :: lda,  lwork, info 
  CHARACTER ( len = 1 ) :: jobz, uplo
  COMPLEX * 16, DIMENSION( : ), ALLOCATABLE :: work
  REAL * 8, DIMENSION( : ), ALLOCATABLE ::  w 
  REAL * 8, DIMENSION( : ), ALLOCATABLE :: rwork

  ! Temporary variables for timing the time to solution of the zheev LAPACK function  
  REAL * 8  :: t_start, t_stop 
  REAL * 8  :: cclock
  EXTERNAL cclock

  ! Minimal instructions for reading the matrix size from standard input 
  arg_count = command_argument_count( )
  IF( arg_count .LT. 1 ) THEN
     WRITE( *, * ) "Sample zheev code."
     WRITE( *, * ) "usage: $EXECUTABLE {matrix size}"
     CALL exit( 1 )
  END IF

  CALL get_command_argument( 1, arg1 )
  READ( arg1 , * ) n 

  ! Allocation of the Hermitian Matrix
  ALLOCATE( A( n, n ) )

  ! Allocation of temporary variables for LAPACK call to ZHEEV workspace  
  ALLOCATE( w( n ) )
  ALLOCATE( work( 1 ) )
  ALLOCATE( rwork( 3 * n - 2 ) )

  ! Initialization of the sample matrix 
  CALL fillMatrix( A, n )

  ! Print the Initliazed Hermitian matrix
  INQUIRE( iolength = reclen ) A
  OPEN( unit = out_unit, file = "matrix.dat", form = "unformatted", access = "direct", RECL = reclen )
  WRITE( out_unit, REC = 1 ) A
  CLOSE( out_unit )


  ! Initialization of the LAPACK zheev parameters
  lda = n
  jobz = "N"
  uplo = "U"
  info = 0
  lwork = -1

  ! Workspace query
  CALL zheev( jobz, uplo, n, A, lda, w, work, lwork, rwork, info ) 
  lwork = work( 1 )
  ! Re-allocation of the workspace accordingly to the obtained lwork dimension
  DEALLOCATE ( work  )
  ALLOCATE ( work( lwork) )

  ! Time measuring for the eingenvalues computation
  t_start = cclock( )
  CALL zheev( jobz, uplo, n, A, lda, w, work, lwork, rwork, info )
  t_stop = cclock( )

  WRITE ( *, 67 ) 'Time to compute eigenvalues for an Hermitian matrix of dimension' &
       , n, ' using LAPACK function ZHEEV: ', t_stop - t_start, '(seconds)' 

  ! Print the eigenvalues of the Hermitian matrix
  INQUIRE( iolength = reclen ) w
  OPEN( unit = out_unit, file = "eigenvalues.dat", form = "unformatted", access = "direct", RECL = reclen )
  WRITE( out_unit, REC = 1 ) w 
  CLOSE( out_unit )

67 FORMAT ( 2X, A30, F30.12 )

  ! Memory Deallocation and Cleaning 
  DEALLOCATE( A )
  DEALLOCATE( w )
  DEALLOCATE( work )
  DEALLOCATE( rwork )

END PROGRAM Diagonalize_random_lapack
